﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WpfGeoTagger.Models
{
	/// <summary>
	/// 編集対象となりえる値の共通インターフェースを表します。
	/// </summary>
	interface IEditValue
	{
		/// <summary>
		/// 編集を開始します。
		/// </summary>
		void BeginEdit();

		/// <summary>
		/// 編集を終了します。
		/// </summary>
		/// <param name="isSave">編集された値を保存する場合は true。破棄する場合は false。</param>
		void FinishEdit( bool isSave );
	}

	/// <summary>
	/// 編集対象となる値を格納します。
	/// </summary>
	/// <typeparam name="T">値の型情報。</typeparam>
	class EditValue< T > : IEditValue
	{
		/// <summary>
		/// 編集を開始します。
		/// </summary>
		public void BeginEdit()
		{
			this._backup = this.Value;
		}

		/// <summary>
		/// 編集を終了します。
		/// </summary>
		/// <param name="isSave">編集された値を保存する場合は true。破棄する場合は false。</param>
		public void FinishEdit( bool isSave )
		{
			// 破棄の場合は値を戻す
			if( !isSave )
			{
				this.Value = this._backup;
				this._backup = default( T );
			}
		}

		/// <summary>
		/// 編集前の値を取得する。
		/// </summary>
		public T Backup { get { return this._backup; } }

		/// <summary>
		/// 設定値を取得または設定します。
		/// </summary>
		public T Value { get; set; }

		/// <summary>
		/// 編集前の設定値。
		/// </summary>
		private T _backup;
	}
}
