﻿using System;
using System.Runtime.InteropServices;

namespace WpfGeoTagger.Models
{
	/// <summary>
	/// Google Map の操作を行う JavaScript に関連付けられるクラスです。
	/// </summary>
	[ComVisible( true )]
	public class MapHost
	{
		/// <summary>
		/// マップの移動が行われた時、スクリプト側から呼び出されます。
		/// </summary>
		/// <param name="latitude">緯度。</param>
		/// <param name="longitude">経度。</param>
		public void OnMapMoved( double latitude, double longitude )
		{
			this.MapChanged( this, new MapChangedEventArgs( MapAction.Moved, latitude, longitude ) );
		}

		/// <summary>
		/// マーカーが追加された時、スクリプト側から呼び出されます。
		/// </summary>
		/// <param name="id">マーカーの識別子。</param>
		/// <param name="latitude">緯度。</param>
		/// <param name="longitude">経度。</param>
		public void OnMarkerAdded( int id, double latitude, double longitude )
		{
			this.MarkerChanged( this, new MarkerChangedEventArgs( MarkerAction.Added, id, latitude, longitude ) );
		}

		/// <summary>
		/// マーカーの住所取得が行われた時、スクリプト側から呼び出されます。
		/// </summary>
		/// <param name="id">マーカーの識別子。</param>
		/// <param name="address">住所。</param>
		public void OnMarkerAddress( int id, string address )
		{
			this.MarkerChanged( this, new MarkerChangedEventArgs( MarkerAction.Address, id, address ) );
		}

		/// <summary>
		/// マーカーの選択状態が変更された時、スクリプト側から呼び出されます。
		/// マーカーの選択は常に単体のみとなります。
		/// </summary>
		/// <param name="id">選択されたマーカーの識別子。</param>
		public void OnMarkerSelected( int id )
		{
			this.MarkerChanged( this, new MarkerChangedEventArgs( MarkerAction.Selected, id ) );
		}

		/// <summary>
		/// マーカーが移動された時、スクリプト側から呼び出されます。
		/// </summary>
		/// <param name="id">マーカーの識別子。</param>
		/// <param name="latitude">緯度。</param>
		/// <param name="longitude">経度。</param>
		public void OnMarkerMoved( int id, double latitude, double longitude )
		{
			this.MarkerChanged( this, new MarkerChangedEventArgs( MarkerAction.Moved, id, latitude, longitude ) );
		}

		/// <summary>
		/// マーカーの移動が完了した時、スクリプト側から呼び出されます。
		/// </summary>
		/// <param name="id">マーカーの識別子。</param>
		public void OnMarkerMoveFinished( int id )
		{
			this.MarkerChanged( this, new MarkerChangedEventArgs( MarkerAction.MoveFinished, id ) );
		}

		/// <summary>
		/// マーカーの移動が開始された時、スクリプト側から呼び出されます。
		/// </summary>
		/// <param name="id">マーカーの識別子。</param>
		public void OnMarkerMoveStarted( int id )
		{
			this.MarkerChanged( this, new MarkerChangedEventArgs( MarkerAction.MoveStarted, id ) );
		}

		/// <summary>
		/// マーカーが削除された時、スクリプト側から呼び出されます。
		/// </summary>
		/// <param name="id">マーカーの識別子。</param>
		public void OnMarkerRemoved( int id )
		{
			this.MarkerChanged( this, new MarkerChangedEventArgs( MarkerAction.Removed, id ) );
		}

		/// <summary>
		/// マップが変更された時に発生します。
		/// </summary>
		public event EventHandler< MapChangedEventArgs > MapChanged;

		/// <summary>
		/// マーカーが変更された時に発生します。
		/// </summary>
		public event EventHandler< MarkerChangedEventArgs > MarkerChanged;
	}
}